import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

public class TestPanel extends JPanel implements Runnable {
	private static final int PWIDTH = 500; // madhsia e panelit
	private static final int PHEIGHT = 400;
	private Thread animator; // pr animim
	private volatile boolean running = false; // ndalon animimin
	private volatile boolean gameOver = false; // terminon lojn

	private Graphics dbg;
	private Image dbImage = null;

	private int period = 10; // shejtsia 100 FPS

	private volatile boolean isPaused = false;

	private int angle = 0;
	private int offset = 3;
	
	public TestPanel() {
		setBackground(Color.WHITE);
		setPreferredSize(new Dimension(PWIDTH, PHEIGHT));
		setFocusable(true);
		requestFocus(); // JPanel tani pranon ngjarje tastesh
		readyForTermination();
		// krijo komponentet e lojs
		// ...
		// dgjo pr shtypjet e mausit
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				testPress(e.getX(), e.getY());
			}
		});
	}

	private void readyForTermination() {
		addKeyListener(new KeyAdapter() {
			// dgjo pr esc, q, end, ctrl-c
			public void keyPressed(KeyEvent e) {
				int keyCode = e.getKeyCode();
				if ((keyCode == KeyEvent.VK_ESCAPE)
						|| (keyCode == KeyEvent.VK_Q)
						|| (keyCode == KeyEvent.VK_END)
						|| ((keyCode == KeyEvent.VK_C) && e.isControlDown())) {
					running = false;
				}
				else if (keyCode == KeyEvent.VK_P) {
					pauseGame();
				}
				else if (keyCode == KeyEvent.VK_R) {
					resumeGame();
				}
			}
		});
	}

	/* A sht (x,y) e rndsishme pr lojn? */
	private void testPress(int x, int y) {
		if (!gameOver) {
			// bj dika
		}
	}

	/*
	 * Prit q ky JPanel t shtohet te JFrame/JApplet para startimit.
	 */
	public void addNotify() {
		super.addNotify(); // krijon peer-in
		startGame(); // starton fijen
	}

	/* Inicializo dhe starto fijen */
	private void startGame() {
		if (animator == null || !running) {
			animator = new Thread(this);
			animator.start();
		}
	}

	/* Invokohet nga shfrytzuesi pr t ndaluar ekzekutimin */
	public synchronized void stopGame() {
		running = false;
		notify();
	}

	/* Prsrit: update, render, sleep ashtu q t zgjas period ms */
	public void run() {
		long beforeTime, timeDiff, sleepTime;
		beforeTime = System.currentTimeMillis();
		running = true;
		while (running) {
			try {
				if (isPaused) {
					synchronized (this) {
						while (isPaused && running)
							wait();
					}
				}
			} catch (InterruptedException e) {
			}
			gameUpdate();
			gameRender();
			paintScreen();
			timeDiff = System.currentTimeMillis() - beforeTime;
			sleepTime = period - timeDiff; // koha e mbetur n iterimin
			if (sleepTime <= 0) // update-render zgjati m tepr
				sleepTime = 5; // megjithat flej pak
			try {
				Thread.sleep(sleepTime); // in ms
			} catch (InterruptedException ex) {
			}
			beforeTime = System.currentTimeMillis();
		}
		System.exit(0);
	}

	/* Azhorno gjendjen e lojs */
	private void gameUpdate() {
		if (!gameOver) {
			// ...
		}
	}

	/* Aktivisht rendero n ekran imazhin e baferit */
	private void paintScreen() {
		Graphics g;
		try {
			g = this.getGraphics(); // merr kontekstin grafik t panelit
			if ((g != null) && (dbImage != null))
				g.drawImage(dbImage, 0, 0, null);
			Toolkit.getDefaultToolkit().sync(); // sync n disa sist.
			g.dispose();
		} catch (Exception e) {
			System.out.println("Gabim n kontekstin grafik: " + e);
		}
	}

	/* Vizato kornizn aktuale n nj bafer imazhi */
	private void gameRender() {
		if (dbImage == null) { // krijo baferin
			dbImage = createImage(PWIDTH, PHEIGHT);
			if (dbImage == null) {
				System.out.println("dbImage sht null");
				return;
			} else {
				dbg = dbImage.getGraphics();
			}
		}
		// pastro prapavin
		dbg.setColor(Color.white);
		dbg.fillRect(0, 0, PWIDTH, PHEIGHT);
		// vizato elementet e lojs
		int centerX = PWIDTH / 2;
		int centerY = PHEIGHT / 2;
		dbg.setColor(Color.RED);
		dbg.fillOval((int)(75 * Math.cos(angle * Math.PI / 180)) + centerX,
				-(int)(75 * Math.sin(angle * Math.PI / 180)) + centerY,
				40, 40);
		angle += offset;
		angle %= 360;
		// ...
		if (gameOver) {
			gameOverMessage(dbg);
		}
	}

	/* Afisho mesazhin game-over */
	private void gameOverMessage(Graphics g) {
		String msg = "Game Over";
		int x = 0, y = 0;
		// Kodi pr llogaritjen e x dhe y...
		g.drawString(msg, x, y);
	}

	public void pauseGame() {
		isPaused = true;
	}

	public synchronized void resumeGame() {
		isPaused = false;
		notify();
	}


	/*
	 * public void paintComponent(Graphics g) { super.paintComponent(g); if
	 * (dbImage != null) { g.drawImage(dbImage, 0, 0, null); } }
	 */

}
